package com.example.mynotepad.MenuFeatures.VoiceMassages;

import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;

import com.example.mynotepad.MenuFeatures.AllNotes.AllNotesActivity;
import com.example.mynotepad.MenuFeatures.CustomToolbarOption;
import com.example.mynotepad.MenuFeatures.CustomToolbarOptionListener;
import com.example.mynotepad.MenuFeatures.VoiceMassages.VoiceMassagesRecyclerView.MyVoice;
import com.example.mynotepad.MenuFeatures.VoiceMassages.VoiceMassagesRecyclerView.OnVoiceMassageClickListener;
import com.example.mynotepad.MenuFeatures.VoiceMassages.VoiceMassagesRecyclerView.VoiceAdaptor;
import com.example.mynotepad.R;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.io.File;
import java.util.ArrayList;

import static com.example.mynotepad.MainActivity.noteDataBase;

public class VoiceMassagesActivity extends AppCompatActivity {
    private FloatingActionButton allVoiceBtn;
    private RecyclerView recyclerViewRecordings;
    private VoiceAdaptor recordingAdapter;
    private ArrayList<MyVoice> myVoiceArrayList;
    private CustomToolbarOption customToolbarOption;
    private MyVoice myChosenVoice;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_voice_massages);
        init();
        fetchRecordings();
        addListener();
    }

 /*   @Override
    protected void onResume() {
        super.onResume();
        init();
    }*/

    private void init() {
        customToolbarOption = findViewById(R.id.customToolbarOption);
        customToolbarOption.setVisibility(View.GONE);
        myVoiceArrayList = new ArrayList<>();
        allVoiceBtn = findViewById(R.id.addVoiceButton);

        /** enabling back button ***/
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        /** setting up recyclerView **/
        recyclerViewRecordings = findViewById(R.id.recyclerView);
        recyclerViewRecordings.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        recyclerViewRecordings.setHasFixedSize(true);


    }

    private void fetchRecordings() {

        File root = android.os.Environment.getExternalStorageDirectory();
        String path = root.getAbsolutePath() + "/VoiceRecorderSimplifiedCoding/Audios";
        Log.d("Files", "Path: " + path);
        File directory = new File(path);
        File[] files = directory.listFiles();
        //  Log.d("Files", "Size: " + files.length);
        if (files != null) {

            for (int i = 0; i < files.length; i++) {

                Log.d("Files", "FileName:" + files[i].getName());
                String fileName = files[i].getName();
                String recordingUri = root.getAbsolutePath() + "/VoiceRecorderSimplifiedCoding/Audios/" + fileName;

                MyVoice myVoice = new MyVoice(recordingUri, fileName, false);
                myVoiceArrayList.add(myVoice);
            }

        }
        recordingAdapter = new VoiceAdaptor(this, myVoiceArrayList);
        recyclerViewRecordings.setAdapter(recordingAdapter);

    }

    private void addListener() {
        allVoiceBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(VoiceMassagesActivity.this, RecorderActivity.class);
                startActivity(intent);
            }
        });

        recordingAdapter.setOnVoiceMassageClickListener(new OnVoiceMassageClickListener() {
            @Override
            public void onItemLongClicked(MyVoice item) {
                customToolbarOption.setVisibility(View.VISIBLE);
                customToolbarOption.hideStar();
                myChosenVoice = item;
            }
        });

        //add listener to custom toolbar option and set being archived or not to be saved in database
        customToolbarOption.setCustomToolbarOptionListener(new CustomToolbarOptionListener() {
            @Override
            public void onStarClicked() {
            }

            @Override
            public void onDeleteClicked() {
                deleteClicked();
            }

            @Override
            public void onCloseClicked() {
                customToolbarOption.setVisibility(View.GONE);
            }
        });
    }

    private void deleteClicked() {
        AlertDialog.Builder alert = new AlertDialog.Builder(VoiceMassagesActivity.this);
        alert.setMessage("are you sure you wanna delete it ??? ");
        alert.setPositiveButton("yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                myVoiceArrayList.remove(myChosenVoice);
                System.out.println(myChosenVoice.getFileName() +"        "+myChosenVoice.getUri() );
                new File(myChosenVoice.getFileName()).delete();
                init();
                fetchRecordings();
                addListener();
            }
        });
        alert.setNegativeButton("no", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                dialogInterface.cancel();
            }
        });
        alert.show();
    }

}

